import request from "request-promise-native";
import { Response } from "request";
import { INewsItem, IAggregator } from "./aggregator";
import { getRepository } from "typeorm";
import { AggroNewsItem } from "@db/entity/AggroNewsItem";
import cheerio from "cheerio";

const urlPattern = /diary\.php\?no=(\d+)/i;
const kissDiaryRoot = "http://www.kisskiss.tv/kiss";
const FEED_NAME = "kisskisstv-diary";

async function aggregate() {
    let repo = getRepository(AggroNewsItem);

    let lastPost = await repo.findOne({
        select: [ "newsId" ],
        where: { feedName: FEED_NAME },
        order: { newsId: "DESC" }
    });

    if(!lastPost)
        lastPost = repo.create({
            newsId: 0
        });
    
    try {
        let mainPageRes = await request(`${kissDiaryRoot}/diary.php`, {resolveWithFullResponse: true}) as Response;
        
        if(mainPageRes.statusCode != 200)
            return [];

        let rootNode = cheerio.load(mainPageRes.body);

        let diaryEntries = rootNode("div.blog_frame_middle ul.disc li a");

        if(diaryEntries.length == 0) {
            console.log("[KISS DIARY] Failed to find listing!");
        }

        let result : INewsItem[] = [];
        let latestEntry = lastPost.newsId;

        for(let a of diaryEntries.get() as CheerioElement[]) {
            let matches = urlPattern.exec(a.attribs.href);
            if(!matches)
                continue;
            
            let id = +matches[1];

            if(id <= lastPost.newsId)
                continue;

            if(id > latestEntry)
                latestEntry = id;

            let diaryLink = `${kissDiaryRoot}/${a.attribs.href}`;
            let res = await request(diaryLink, {resolveWithFullResponse: true}) as Response;

            if(res.statusCode != 200)
                continue;

            let node = cheerio.load(res.body);

            let title = node("table.blog_frame_top tr td a");
            let contents = node("div.blog_frame_middle");
            let bottomFrame = contents.find("div.blog_data");
            bottomFrame.remove();

            result.push({
                newsId: id,
                feedId: FEED_NAME,
                link: diaryLink,
                title: title.text(),
                author: "KISS BLOG",
                contents: contents.html(),
                embedColor: 0xf4c100,
                needsTranslation: true
            });
        }

        return result;
    } catch(err) {
        return [];
    }
}

export default {
    aggregate: aggregate
} as IAggregator;